@@ -1,8 +1,9 @@
-    # -*- coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 import logging
 
 from django.contrib.auth.models import AnonymousUser
 
+from request_profiler import settings
 from request_profiler.models import RuleSet, ProfilingRecord
 from request_profiler.signals import request_profile_complete
 
@@ -33,6 +34,13 @@ class ProfilingMiddleware(object):
 
     def process_view(self, request, view_func, view_args, view_kwargs):
         """Add view_func to the profiler info."""
+        # force the creation of a valid session by saving it.
+        if (
+            hasattr(request, 'session') and
+            request.session.session_key is None and
+            settings.STORE_ANONYMOUS_SESSIONS is True
+        ):
+            request.session.save()
         request.profiler.view_func_name = view_func.__name__
 
     def process_response(self, request, response):
@@ -49,6 +57,13 @@ class ProfilingMiddleware(object):
         assert getattr(request, 'profiler', None) is not None, (
             u"Request has no profiler attached."
         )
+
+        # call the global exclude first, as there's no point continuing if this
+        # says no.
+        if settings.GLOBAL_EXCLUDE_FUNC(request) is False:
+            del request.profiler
+            return response
+
         # see if we have any matching rules
         rules = self.match_rules(request, RuleSet.objects.live_rules())
 
@@ -71,5 +86,6 @@ class ProfilingMiddleware(object):
         # if any signal receivers have called cancel() on the profiler, then
         # this method will _not_ save it.
         profiler.capture()
+        profiler.set_response(response)
 
         return response
